<?php

/**
 * @file
 * Contains \Drupal\mcapi\Entity\Wallet.
 */

namespace Drupal\mcapi\Entity;

use Drupal\mcapi\Exchange;
use Drupal\mcapi\Entity\WalletInterface;
use Drupal\user\EntityOwnerInterface;
use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Cache\Cache;
use Drupal\Core\Field\BaseFieldDefinition;

/**
 * Defines the wallet entity.
 * The metaphor is not perfect because this wallet can hold a negaive balance.
 * A more accurate term, 'account', has another meaning in Drupal
 *
 * @note Wallet 'holders' could be any content entity, but the wallet owner is
 * always a user, typically the owner of the holder entity
 *
 * @note The canonical link is actually a view
 *
 *
 * @ContentEntityType(
 *   id = "mcapi_wallet",
 *   label = @Translation("Wallet"),
 *   module = "mcapi",
 *   handlers = {
 *     "view_builder" = "Drupal\mcapi\ViewBuilder\WalletViewBuilder",
 *     "storage" = "Drupal\mcapi\Storage\WalletStorage",
 *     "storage_schema" = "Drupal\mcapi\Storage\WalletStorageSchema",
 *     "access" = "Drupal\mcapi\Access\WalletAccessControlHandler",
 *     "form" = {
 *       "edit" = "Drupal\mcapi\Form\WalletForm",
 *       "create" = "Drupal\mcapi\Form\WalletAddForm",
 *       "delete" = "Drupal\Core\Entity\ContentEntityDeleteForm",
 *     },
 *     "views_data" = "Drupal\mcapi\Views\WalletViewsData",
 *     "route_provider" = {
 *       "html" = "Drupal\mcapi\Entity\WalletRouteProvider",
 *     },
 *   },
 *   admin_permission = "configure mcapi",
 *   base_table = "mcapi_wallet",
 *   label_callback = "Drupal\mcapi\ViewBuilder\WalletViewBuilder::DefaultLabel",
 *   entity_keys = {
 *     "label" = "name",
 *     "id" = "wid",
 *     "uuid" = "uuid"
 *   },
 *   translatable = FALSE,
 *   links = {
 *     "canonical" = "/wallet/{mcapi_wallet}",
 *     "log" = "/wallet/{mcapi_wallet}/log",
 *     "inex" = "/wallet/{mcapi_wallet}/inex",
 *     "delete-form" = "/wallet/{mcapi_wallet}/delete",
 *     "edit-form" = "/wallet/{mcapi_wallet}/manage"
 *   },
 *   field_ui_base_route = "mcapi.admin_wallets"
 * )
 */
class Wallet extends ContentEntityBase implements WalletInterface {
  
  const ACCESS_ANY = 'a';//anonymous users
  const ACCESS_MEMBERS = 'm';//authenticated users
  const ACCESS_USERS = 'u';//refer to wallet_access table
  const ACCESS_OG = 'g';//same groups as wallet owner
  const ACCESS_OWNER = 'o';//in memory only - this is saved as u with the wallet owner 
  
  const OP_DETAILS = 'details';
  const OP_SUMMARY = 'summary';
  const OP_PAYIN = 'payin';
  const OP_PAYOUT = 'payout';
  const OP_MANAGE = 'manage';
  
  private $holder;
  private $stats = [];
  private $access = [];

  /**
   * {@inheritDoc}
   */
  public function getHolder() {
    if (!isset($this->holder)) {
      $this->holder = $this->entityTypeManager()
        ->getStorage($this->entity_type->value)
        ->load($this->pid->value);
      //in the impossible event that there is no owner, set
      if (!$this->holder) {
        throw new \Exception('Holder of wallet ' . $this->id() . ' does not exist: ' . $this->entity_type->value . ' ' . $this->pid->value);
      }
    }
    return $this->holder;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    $holder = $this->getHolder();
    return $holder instanceof \Drupal\user\UserInterface ?
      $holder :
      //because all wallet holder, whatever entity type, implement OwnerInterface
      $holder->getOwner();
  }
  /**
   * {@inheritdoc}
   */
  public function setHolder(EntityOwnerInterface $entity) {
    $this->pid = $entity->id();
    $this->entity_type = $entity->getEntityTypeId();
  }

  /**
   * {@inheritdoc}
   */
  public static function heldBy(ContentEntityInterface $entity) {
    return \Drupal::entityTypeManager()
      ->getStorage('mcapi_wallet')
      ->filter(['holder' => $entity]);
  }


  /**
   * {@inheritdoc}
   */
  public function label($langcode = NULL, $full = TRUE) {
    return call_user_func($this->getEntityType()->getLabelCallback(), $this);
  }

  /**
   * {@inheritdoc}
   */
  public static function preCreate(EntityStorageInterface $storage_controller, array &$values) {
    if (!array_key_exists('pid', $values) && array_key_exists('entity_type', $values)) {
      throw new Exception("new wallets must have an entity_type and a parent entity_id (pid)");
    }
    $values += [
      'name' => '', 
      'orphaned' => 0, 
      'created' => REQUEST_TIME
    ];
  }
  
  /**
   * {@inheritdoc}
   */
  public function postCreate(\Drupal\Core\Entity\EntityStorageInterface $storage) {
    //put the default values for the access here
    //@todo inject settings
    $settings = \Drupal::config('mcapi.settings');
    //at the point of creation the wallet has no settings of its own, so we use the defaults from settings
    //many defaults are possible so we try them in order of caution
    foreach (array_keys(Exchange::walletOps()) as $op) {
      $setting = $settings->get($op);
      foreach (['a', 'm', 'g', 'o', 'u'] as $p) {//@todo use the constants
        if (!empty($setting[$p])) {
          $val = $p;
          if ($p === 'o') {
            $val = \Drupal::currentUser()->id();
          }
          $this->{$op}->value = $val;
          continue 2;
        }
      }
    }
    //if (empty($this->name->value)) {
      //$this->name->value == '';
    //}
  }

  /**
   * {@inheritdoc}
   */
  public function doSave($id, EntityInterface $entity) {
    //check the name length
    if (strlen($this->name->value) > 64) {
      $this->name->value = substr($this->name->value, 0, 64);
      drupal_set_message(t(
          'Wallet name was truncated to 64 characters: !name', array('!name' => $this->name->value))
        , 'warning');
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields['wid'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Wallet ID'))
      ->setDescription(t('The unique wallet ID.'))
      ->setReadOnly(TRUE)
      ->setSetting('unsigned', TRUE);

    $fields['uuid'] = BaseFieldDefinition::create('uuid')
      ->setLabel(t('UUID'))
      ->setDescription(t('The wallet UUID.'))
      ->setReadOnly(TRUE);
    $fields['name'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Wallet name'))
      ->setDescription(t("Set by the wallet's owner"))
      ->setSetting('max_length', 64)
      ->setSetting('default', '')
      ->setDefaultValue('');//if we leave the default to be NULL it is difficult to filter with mysql

    $fields['entity_type'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Holder entity type'))
      ->setDescription(t('The timezone of this user.'))
      ->setSetting('max_length', 32)
      ->setRequired(TRUE);

    $fields['pid'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Holder entity ID'));

    foreach (Exchange::walletOps() as $key => $info) {
      $fields[$key] = BaseFieldDefinition::create('string')
        ->setLabel($info[0])
        ->setDescription($info[1])
        ->setSetting('length', 1);
    }
    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created on'))
      ->setDescription(t('The time that the wallet was created.'))
      ->setRevisionable(FALSE);
    $fields['orphaned'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Orphaned'))
      ->setDescription(t('TRUE if this wallet is not currently held'))
      ->setDefaultValue(0);
    return $fields;
  }

  /**
   * {@inheritDoc}
   */
  public function currenciesUsed() {
    if (!$this->currencies_used) {
      $this->currencies_used = [];
      foreach (Currency::loadMultiple(array_keys($this->getSummaries())) as $currency) {
        $this->currencies_used[$currency->id()] = $currency;
      }
    }

    return $this->currencies_used;
  }

  /**
   * {@inheritDoc}
   */
  public function getSummaries() {
    if (!$this->stats) {
      $this->stats = $this->entityTypeManager()
        ->getStorage('mcapi_transaction')
        ->summaryData($this->id());
      //ensure there is a value for all available currencies.
      foreach (Exchange::currenciesAvailable($this) as $curr_id => $currency) {
        if (!array_key_exists($curr_id, $this->stats)) {
          $this->stats[$curr_id] = array(
            'balance' => 0,
            'trades' => 0,
            'volume' => 0,
            'gross_in' => 0,
            'gross_out' => 0,
            'partners' => 0
          );
        }
      }
    }
    return $this->stats;
  }

  /**
   * {@inheritdoc}
   */
  public function getStats($curr_id) {
    $summaries = $this->getSummaries();
    if (array_key_exists($curr_id, $summaries)) {
      return $summaries[$curr_id];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getStat($curr_id, $stat) {
    $stats = getStats($curr_id);
    if (array_key_exists($curr_id, $stats)) {
      return $stats[$curr_id];
    }
  }

  /**
   * {@inheritDoc}
   */
  public function history($from = 0, $to = 0) {
    $conditions = array(
      'involving' => $this->id()
    );
    if ($from) {
      $conditions['from'] = $from;
    }
    if ($to) {
      $conditions['to'] = $to;
    }
    return $this->entityTypeManager()->getStorage('mcapi_transaction')->filter($conditions);
  }

  /**
   * {@inheritDoc}
   */
  public static function orphan(ContentEntityInterface $holder) {
    $new_holder_entity = Exchange::findNewHolder($holder);
    $wallet_ids = \Drupal::entityTypeManager()
      ->getStorage('mcapi_wallet')
      ->filter(['holder' => $holder]);
    foreach (Self::loadMultiple($wallet_ids) as $wallet) {
      $criteria = ['involving' => $wallet->id()];
      if (\Drupal::entityTypeManager()->getStorage('mcapi_transaction')->filter($criteria)) {
        $new_name = t(
          "Formerly !name's wallet: !label", ['!name' => $wallet->label(), '!label' => $wallet->label(NULL, FALSE)]
        );
        $wallet->set('name', $new_name)
          ->set('entity_type', $new_holder_entity->getEntityTypeId())
          ->set('pid', $new_holder_entity->id())
          ->save();
        //@note this implies the number of wallets an exchange can own to be unlimited.
        //or more likely that this max isn't checked during orphaning
        drupal_set_message(t(
          "!name's wallets are now owned by @entity_type !entity_label", [
          '!name' => $wallet->label(),
          '!exchange' => \Drupal::l($new_holder_entity->label(), $exchange->url())
            ]
        ));
        \Drupal::logger('mcapi')->notice(
          'Wallet @wid was orphaned to @entitytype @id', [
          '@wid' => $wallet->id(),
          '@entitytype' => $new_holder_entity->getEntityTypeId(),
          '@id' => $new_holder_entity->id()->id()
          ]
        );
      }
      else {
        $wallet->delete();
        return;
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  public function invalidateTagsOnSave($update) {
    parent::invalidateTagsOnSave($update);
    //invalidate tags for the previous and current parents
    $tags = [
      $this->entity_type->value . ':' . $this->pid->value
    ];
    Cache::invalidateTags($tags);
  }

}
